package net.sf.brunneng.fusie.core

import net.sf.brunneng.fusie.core.expressions.Expression

/**
 * User: goodg_000
 * Date: 31.03.13
 * Time: 18:43
 */
class ProbableValue[T](val value : T, var probability : Expression[Boolean]) {

  def calcProbabilityValue: Double = {
    if (probability == null)
      return 1.0

    val variables = probability.getAllVariables.groupBy(_.varName).map(_._2.head).toArray
    val probabilities = variables.map(v => v.probableValue.value.asInstanceOf[Double])

    var res = 0.0
    val len = variables.size
    def generateEvents(i: Int, p: Double) {
      if (i >= len) {
        if (probability.calculate) {
          res += p
        }
      }
      else {
        val v = variables(i)
        val nextP = p*probabilities(i)
        v.setValue(true)
        generateEvents(i+1, nextP)

        v.setValue(false)
        generateEvents(i+1, nextP)
      }
    }

    generateEvents(0, 1.0)

    variables.zip(probabilities).foreach(p => p._1.setValue(p._2))
    res
  }

  def equalValue(other : ProbableValue[T]) : Boolean = {
    if (value.isInstanceOf[Double]) {
      val v1 = value.asInstanceOf[Double]
      val v2 = other.value.asInstanceOf[Double]
      scala.math.abs(v1 - v2) < Constants.EPSILON
    }

    value.equals(other.value)
  }
}
