package net.sf.brunneng.fusie.core

import net.sf.brunneng.fusie.core.expressions.{F2ExpressionFactory, VariableExpression, ValueExpression, Expression}
import net.sf.brunneng.fusie.core.statements.AssignmentStatement
import scala.collection.mutable

/**
 * User: goodg_000
 * Date: 29.03.13
 * Time: 23:58
 */
class Rule(val preconditions: List[Expression[Boolean]],
           val conclusions: List[AssignmentStatement[Any]],
           val probability: Expression[Double] = new ValueExpression[Double](1.0)) {

  var name: String = null
  var index = 0

  val internalId = this.toString
  var allVariablesCache: List[Variable[Any]] = null

  def this(name: String,
           preconditions: List[Expression[Boolean]],
           conclusions: List[AssignmentStatement[Any]],
           probability: Expression[Double] = new ValueExpression[Double](1.0)) {
    this(preconditions, conclusions, probability)
    this.name = name
  }

  def isReadyForExecution : Boolean = collectAllVariables.forall(v => v.isDefined)

  def allPreconditionsAreTrue: Boolean = preconditions.forall(p => p.calculate)

  def executeConclusions(probabilityContext: Context) {
    val p = probability.calculate
    var pExpr = probability.getProbabilityExpression

    if (p < 1.0) {
      val v = new Variable[Boolean](internalId, classOf[Boolean])
      v.env = new Environment
      v.env.pushContext(probabilityContext)

      pExpr = F2ExpressionFactory.andNullSafe(pExpr, new VariableExpression[Boolean](v))
      probabilityContext.set(internalId, new ProbableValue[Any](p, null))
    }
    else {
      probabilityContext.set(internalId, null)
    }

    pExpr = preconditions.map(_.getProbabilityExpression).foldLeft(pExpr)((curr, next) =>
      F2ExpressionFactory.andNullSafe(curr, next))

    conclusions.foreach(a => {
      a.evaluate()
      a.variable.probableValue.probability = F2ExpressionFactory.andNullSafe(a.expr.getProbabilityExpression, pExpr)
    })
  }

  def conclusionsHasNoConflicts : Boolean = {
    conclusions.forall(a => !a.variable.isDefined ||
      a.variable.probableValue.equalValue(new ProbableValue[Any](a.expr.calculate, null)))
  }

  def collectAllVariables: List[Variable[Any]] = {
    if (allVariablesCache != null) {
      return allVariablesCache
    }

    val res = new mutable.ListBuffer[Variable[Any]]()

    preconditions.foreach(expr => res ++= expr.getAllVariables)
    conclusions.foreach(stmt => res ++= stmt.expr.getAllVariables)
    res ++= probability.getAllVariables

    allVariablesCache = res.toList
    allVariablesCache
  }

  def collectAllNotDefinedVariables: List[Variable[Any]] =
    collectAllVariables.filter(v => !v.isDefined)

  def collectAssignedVariables: List[Variable[Any]] = conclusions.map(c => c.variable).toList
}
