package net.sf.brunneng.fusie.core.expressions

import net.sf.brunneng.fusie.core.Variable
import collection.mutable.ListBuffer

/**
 * Function on 2 expressions.
 *
 * User: goodg_000
 * Date: 30.03.13
 * Time: 19:35
 */
class F2Expression[VT1, VT2, T](val expr1 : Expression[VT1], val expr2 : Expression[VT2], val op : (VT1, VT2) => T,
                                val funcName : String) extends Expression[T]
  {
  def calculate: T = op(expr1.calculate, expr2.calculate)

  def collectVariables(res : ListBuffer[Variable[Any]]) {
    expr1.collectVariables(res)
    expr2.collectVariables(res)
  }

  def getChildExpressions: List[Expression[Any]] = List(expr1, expr2)

  def getProbabilityExpression: Expression[Boolean] = {
    val probExpr1 = expr1.getProbabilityExpression
    val probExpr2 = expr2.getProbabilityExpression

    funcName match {
      case "||" => F2ExpressionFactory.orNullSafe(probExpr1, probExpr2)
      case _ => F2ExpressionFactory.andNullSafe(probExpr1, probExpr2)
    }
  }

  override def toString: String = "F2Expression(" + expr1 + " " + funcName + " " + expr2 + ")"
}
