package net.sf.brunneng.fusie.input

import net.sf.brunneng.fusie.core.{Variable, Rule}
import net.sf.brunneng.fusie.input.variables.VariableDefinition
import scala.collection.mutable
import net.sf.brunneng.fusie.core.statements.AssignmentStatement
import scala.collection.mutable.ListBuffer
import net.sf.brunneng.fusie.core.expressions.Expression
import net.sf.brunneng.fusie.input.ast.operations.TypeUnsafeOp

/**
 * User: goodg_000
 * Date: 21.04.13
 * Time: 16:40
 */
class ProblemDefinition(var variableDefinitions: List[VariableDefinition[Any]],
                                 var rules: List[Rule], val target: String) {

  def getDefinitionsMap : Map[String, VariableDefinition[Any]] = {
    val varDefinitionsMap = new mutable.HashMap[String, VariableDefinition[Any]]
    variableDefinitions.foreach(vd => varDefinitionsMap(vd.name) = vd)
    varDefinitionsMap.toMap
  }

  def getAllVariables: List[Variable[Any]] = {
    val allVariables = new mutable.ListBuffer[Variable[Any]]()
    rules.foreach(r => allVariables ++= r.collectAllVariables)
    allVariables.toList
  }

  def getNotAssignedAndNotDefinedVariables: List[Variable[Any]] = {
    val definedVariables = variableDefinitions.map(vd => vd.name).toSet
    val assignedVariables = rules.flatMap(r => r.conclusions.map(as => as.variable.varName)).toSet
    val addedVariables = new mutable.HashSet[String]()

    getAllVariables.filter(v => {
      val name = v.varName
      val res = !definedVariables.contains(name) && !assignedVariables.contains(name) && !addedVariables.contains(name)
      addedVariables += name
      res
    })
  }

  def getAllVariablesWithAssigned : List[Variable[Any]] = {
    val allVariables = new mutable.ListBuffer[Variable[Any]]()
    rules.foreach(r => {
      allVariables ++= r.collectAllVariables
      allVariables ++= r.conclusions.map(c => c.variable)
    })
    allVariables.toList
  }

  def getVariablesWithUndefinedTypes : List[Variable[Any]] = {
    getAllVariables.filter(variableWithUndefinedTypeP)
  }

  def getAllConclusions : List[AssignmentStatement[Any]] = {
    val res = new ListBuffer[AssignmentStatement[Any]]
    rules.foreach(r => res ++= r.conclusions )
    res.toList
  }

  def getConclusionsWithUndefinedTypes : List[AssignmentStatement[Any]] = {
    getAllConclusions.filter(c => variableWithUndefinedTypeP(c.variable))
  }

  def getConclusionsWithUndefinedExpressions : List[AssignmentStatement[Any]] = {
    getAllConclusions.filter(c => hasTypeUnsafeExpressionsP(c.expr))
  }

  private def hasTypeUnsafeExpressionsP : Expression[Any] => Boolean =
    e => e.isInstanceOf[TypeUnsafeOp] || !e.getChildExpressions.find(hasTypeUnsafeExpressionsP).isEmpty

  def variableWithUndefinedTypeP : Variable[Any] => Boolean = v => v.clazz.equals(classOf[Any])
  def variableWithDefinedTypeP : Variable[Any] => Boolean = v => !v.clazz.equals(classOf[Any])
}
