package net.sf.brunneng.fusie.input.functions

import java.lang.reflect.{Modifier, Method}
import scala.collection.mutable
import scala.collection.mutable.ListBuffer

/**
 * Created with IntelliJ IDEA.
 * User: evvo
 * Date: 5/22/13
 * Time: 2:52 PM
 * To change this template use File | Settings | File Templates.
 */
class DefaultFunctionsSource extends FunctionsSource {

  private var initializedDefaultSet = false
  private val functionsMap = new mutable.HashMap[String, ListBuffer[Function]]
  private val allSignatures = new mutable.HashSet[FunctionSignature]()

  def tryInit() {
    if (!initializedDefaultSet) {
      registerPublicStaticMethods(classOf[java.lang.Math])
      initializedDefaultSet = true
    }
  }

  def registerFunction(name: String, func: Function) {
    val signature = func.getSignature(name)
    if (allSignatures.contains(signature)) {
      throw new IllegalArgumentException("Function with signature " + signature + " already registered.")
    }
    allSignatures += signature

    var allFunctions = functionsMap.getOrElse(name, null)
    if (allFunctions == null) {
      allFunctions = new ListBuffer[Function]
      functionsMap(name) = allFunctions
    }

    allFunctions += func
  }

  def registerPublicNonStaticMethods(objectName: String, obj: Object) {
    obj.getClass.getMethods.foreach(m => {
      if (!Modifier.isStatic(m.getModifiers()) && !m.getDeclaringClass.equals(classOf[Object])) {
        registerFunction(objectName + "." + m.getName, new Function(obj, m))
      }
    })
  }

  def registerPublicStaticMethods(clazz: Class[_]) {
    clazz.getMethods.foreach(m => {
      if (Modifier.isStatic(m.getModifiers()) && !m.getDeclaringClass.equals(classOf[Object])) {
        registerFunction(clazz.getSimpleName + "." + m.getName, new Function(null, m))
      }
    })
  }

  def getFunctions(name: String): List[Function] = {
    tryInit()
    functionsMap.getOrElse(name, new ListBuffer[Function]).toList
  }
}
