package net.sf.brunneng.fusie.core.expressions.fuzzy

import net.sf.brunneng.fusie.core.expressions.{F2ExpressionFactory, ValueExpression, Expression}
import scala.collection.mutable.ListBuffer
import net.sf.brunneng.fusie.core.Variable

/**
 * Created with IntelliJ IDEA.
 * User: evvo
 * Date: 5/29/13
 * Time: 5:17 PM
 * To change this template use File | Settings | File Templates.
 */
abstract class AbstractMembershipFunction(val testExpression: Expression[Double], val name: String,
                                 val probabilitySource: MembershipFunctionProbabilitySource,
                                  args: List[Expression[Double]]) extends Expression[Double] {

  def calculate: Double = {
    val points = args.map(_.calculate).toArray

    if (points.size < 2) {
      throw new IllegalStateException("Membership function " + this + " should have at least 2 points")
    }

    if (points.size != probabilitySource.shape.size + 1) {
      throw new IllegalStateException("Count of membership function " + this +
        " points (actual " + points.size + ") should be 1 more then count of shape parts (actual " +
        probabilitySource.shape.size + ")" )
    }

    var prev = points.head
    points.tail.foreach(p => {
      if (p < prev) {
        throw new IllegalStateException("Points " + points.toList + " of membership function " + this +
          " should go in ascending order")
      }
      prev = p
    })

    var i = 0
    calculateInternal(testExpression.calculate,
      points.map(v => {
        val p = probabilitySource.getProbability(i)
        val point = points(i)
        i += 1
        (p, point)
      }).toArray)
  }

  def calculateInternal(value: Double, probabilityToValueData: Array[(Double, Double)]): Double

  def collectVariables(res: ListBuffer[Variable[Any]]) {
    testExpression.collectVariables(res)
    args.foreach(_.collectVariables(res))
  }

  def getChildExpressions: List[Expression[Any]] = testExpression :: args

  def getProbabilityExpression: Expression[Boolean] = {
    val children = getChildExpressions
    children.tail.foldLeft(children.head.getProbabilityExpression)((curr, next) =>
      F2ExpressionFactory.andNullSafe(curr, next.getProbabilityExpression))
  }
}
